/*
Copyright Ideaconsult (C) 2005-2007 
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

package chemCast.data;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import javax.help.UnsupportedOperationException;

import org.openscience.cdk.ChemObject;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IChemObject;
import org.openscience.cdk.interfaces.IChemObjectBuilder;
import org.openscience.cdk.interfaces.IChemObjectChangeEvent;
import org.openscience.cdk.interfaces.IChemObjectListener;
import org.openscience.cdk.io.listener.IReaderListener;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.PropertyTranslator;
import ambit.data.molecule.SortedPropertyList;
import ambit.io.RandomAccessReader;
import ambit.io.RandomAccessSDFReader;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.SmilesGeneratorProcessor;
import chemCast.events.DatasetChangeEvent;

public class MoleculesFile extends ChemObject implements IChemObjectListener, IAtomContainersList {
	/**
	 * 
	 */
    protected static String NA = "";
	private static final long serialVersionUID = 4566523260890189354L;
	protected RandomAccessReader reader;
	protected final Hashtable[] updatedProperties;
	private IAtomContainer recordLoaded = null;
	private int recordLoadedIndex = -1;
	protected int selectedIndex = -1;
	protected boolean bufferProperties = true;
    protected PropertyChangeSupport propertyChangeSupport ;
    protected Hashtable<String,String> propertyTranslator;
    protected PropertyTranslator availableProperties;
    protected SortedPropertyList sortedProperties;

	public MoleculesFile(File file,IChemObjectBuilder builder) throws Exception {
    	this(file,builder,null);
    }
	public MoleculesFile(File file,IChemObjectBuilder builder,IReaderListener listener) throws Exception {
		super();
//		if (file.length() < 2000000) throw new Exception("File too small :)");
        propertyChangeSupport =new PropertyChangeSupport(this);
		this.reader = createReader(file, builder,listener);
		reader.readRecord(0);
		updatedProperties = new Hashtable[reader.size()];
		propertyTranslator = new Hashtable<String,String>();
		reader.removeChemObjectIOListener(listener);
        sortedProperties = new SortedPropertyList("");
        availableProperties = new PropertyTranslator();
	}
    public RandomAccessReader createReader(File file,IChemObjectBuilder builder,IReaderListener listener) throws Exception {
        if (file.getName().toLowerCase().endsWith(".sdf"))
            return new RandomAccessSDFReader(file, builder,listener);
        else throw new Exception("Unsupported file format! "+file.getName());
    }
    public void close() {
        sortedProperties.clear();
        if (recordLoaded != null) recordLoaded.removeListener(this);
        if (reader != null)
            try {
                propertyChangeSupport.firePropertyChange(DatasetChangeEvent.op_clear, null, reader);
                
            reader.close();
            } catch (Exception x) {
                x.printStackTrace();
            }
        reader = null;
    }
    
	@Override
	protected void finalize() throws Throwable {
	    close();
		super.finalize();
	}
	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#first()
	 */
	public Object first() {
		return setSelectedIndex(0);
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#last()
	 */
	public Object last() {
		return setSelectedIndex(getAtomContainerCount()-1);
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#setSelectedIndex(int)
	 */
	public Object setSelectedIndex(int index) {
        /*
        firePropertyChange(new SelectItemEvent(this,
                toString(),
                new SelectedItem(null,this,selectedIndex), 
                new SelectedItem(null,this,fpIndex)));
        selectedIndex = fpIndex;
        */
        selectedIndex = index;
		return getAtomContainer(selectedIndex);
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#getSelectedIndex()
	 */
	public int getSelectedIndex() {
		return selectedIndex;
	}

	public void setAtomContainers(IAtomContainer[] molecules) {
		throw new UnsupportedOperationException("setAtomContainers");

	}
	/*
	public IAtomContainer getAtomContainer(int fpIndex) {
		try {
			return (IAtomContainer) reader.readRecord(fpIndex);
		} catch (Exception x) {
			return null;
		}
	}
	
	*/
	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#getAtomContainer(int)
	 */
	public IAtomContainer getAtomContainer(int index) {
		try {
			if (recordLoadedIndex != index) {
				if (recordLoaded != null) {
					recordLoaded = null;
				}
				
				recordLoaded = (IAtomContainer) reader.readRecord(index);
				//recordLoaded.addListener(this);
				recordLoadedIndex = index;
			}
            //if (bufferProperties && (updatedProperties[index]==null)) updatedProperties[index] =  new Hashtable();
			if (updatedProperties[index] != null) {
                Hashtable p = recordLoaded.getProperties();
                Enumeration keys = p.keys();
                //this effectively does buffering of properties
                //Hashtable newp = new Hashtable();
                while (keys.hasMoreElements()) {
                    Object key = keys.nextElement();
                    if ((key ==null) || ("".equals(key))) return null;
                    Object newKey = propertyTranslator.get(key);
                    if (newKey==null) newKey = key;
                    Object oldValue = p.get(key);
                    p.remove(key);
                    p.put(newKey, oldValue);
                }
                p.putAll(updatedProperties[index]);
                //recordLoaded.setProperties(p);
				//recordLoaded.getProperties().putAll(updatedProperties[index]);
            }    

			return recordLoaded;
			
		} catch (Exception x) {
			x.printStackTrace();
			return null;
		}
	}
	

	public void removeAllAtomContainers() {
		// TODO Auto-generated method stub
        sortedProperties.clear();
	}

	public void addAtomContainer(IAtomContainer a) {
		// TODO Auto-generated method stub
        sortedProperties.clear();
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#getAtomContainerCount()
	 */
	public int getAtomContainerCount() {
		if (reader ==null) return 0;
		else
		return reader.size();
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#hasNext()
	 */
	public boolean hasNext() {
		if (reader == null) return false;
		return reader.hasNext();
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#next()
	 */
	public Object next() {
		return setSelectedIndex(getSelectedIndex()+1);
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#hasPrevious()
	 */
	public boolean hasPrevious() {
		if (reader == null) return false;
		return reader.hasPrevious();
	}

	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#previous()
	 */
	public Object previous() {
		return setSelectedIndex(getSelectedIndex()-1);
	}

	public int nextIndex() {
		return reader.nextIndex();
	}

	public int previousIndex() {
		return reader.previousIndex();
	}
	
	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#getProperties(int)
	 */
	public Hashtable getProperties(final int index) {
		if (updatedProperties[index] == null)
			return getAtomContainer(index).getProperties();
		else {
			Hashtable h = new Hashtable();
			h.putAll(updatedProperties[index]);
			h.putAll(getAtomContainer(index).getProperties());
			return h;
		}
	}
	public boolean isBuffered(final int index, final Object key) {
		if (updatedProperties[index] != null) {
			return updatedProperties[index].get(key) != null;
		} else return false;	
	}
	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#getProperty(int, java.lang.Object)
	 */
	public Object getProperty(final int index, final Object key) {
        if ((key ==null) || ("".equals(key.toString()))) return null;
        String newKey = propertyTranslator.get(key.toString());
        if (newKey==null) newKey = key.toString();
		Object o = null;
		if (updatedProperties[index] != null)
			o = updatedProperties[index].get(newKey);
		if (o == null) {
            //System.out.println("Reading record to get Property "+key + " " + index);
			IAtomContainer c = getAtomContainer(index);
			if (c ==null) return null;
			else {
				Object value = c.getProperty(newKey);
                if (bufferProperties) {
                    if (value == null) value = NA;
                    else
	                    try {
	                    	Double dvalue = new Double(value.toString());
	                    	value = dvalue;
	                    } catch (Exception x) {
	                    	
	                    }
                    
                    setProperty(index, key, value);
                }
				return value;
			}
		}
		else return o;
	}
	/* (non-Javadoc)
	 * @see chemCast.similarity.data.IAtomContainersList#setProperty(int, java.lang.Object, java.lang.Object)
	 */
	public final void setProperty(final int index, final Object key, final Object value) {
        if (value == null) return; 
        Object newKey = propertyTranslator.get(key);
        if (newKey==null) newKey = key;
		if (updatedProperties[index] == null) updatedProperties[index] = new Hashtable();
        
        updatedProperties[index].put(newKey,value);
        
	}
	public void stateChanged(IChemObjectChangeEvent event) {
		System.out.println(((IChemObject) event.getSource()).getProperties());
		
	}
    @Override
    public String toString() {
        return reader.toString();
    }
    public void set(Object o) {
    	throw new UnsupportedOperationException("Modifying objects not supported!");
    	
    }
    public void remove() {
    	throw new UnsupportedOperationException("Removing objects not supported!");
    	
    }
    public void add(Object o) {
    	throw new UnsupportedOperationException("Adding objects not supported!");
    	
    }
	public boolean isBufferProperties() {
		return bufferProperties;
	}
	public void setBufferProperties(boolean bufferProperties) {
		this.bufferProperties = bufferProperties;
	}
	
	public void bufferedProperties(List properties) {
		return;
		/*
	    for (int i=0; i < getAtomContainerCount();i++) {
            //System.out.println("buffer");
	    	IAtomContainer c = getAtomContainer(i);
	    	if (c == null) continue;
	    	Hashtable props = c.getProperties();
            
	    	for (int l=0;l<properties.size();l++)  {
	    		Object key = properties.get(l);
	    		Object value = props.get(key);
	    		try {
	    			Double d = new Double(value.toString());
                    value = d;
	    		} catch (Exception x) {	

	    		}
                if (value == null) value = NA;
                    setProperty(i, key, value);
	    	}
	    }
	    */	
	}
	@Override
	public boolean equals(Object obj) {
	    if (obj instanceof MoleculesFile)
	    return obj.toString().equals(toString());
        else return false;
	}
    public synchronized RandomAccessReader getReader() {
        return reader;
    }
    public synchronized void addPropertyChangeListener(
            PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }
    public synchronized void removePropertyChangeListener(
            PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }
    public synchronized void addPropertyChangeListener(String propertyName,
            PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(propertyName,listener);
    }
    public synchronized void removePropertyChangeListener(String propertyName,
            PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(propertyName,listener);
    }
    public void firePropertyChange(String propertyName, 
            Object oldValue, Object newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    public void firePropertyChange(PropertyChangeEvent event) {
        propertyChangeSupport.firePropertyChange(event);
    }
	public Hashtable<String, String> getPropertyTranslator() {
		return propertyTranslator;
	}
	public void setPropertyTranslator(Hashtable<String, String> propertyTranslator) {
		this.propertyTranslator = propertyTranslator;
	}
    public PropertyTranslator getAvailableProperties() {
		return availableProperties;
	}
    public void clearAvailableProperties() {
        this.availableProperties.clear();
        updateTranslator();
        
    }
	public void addAvailableProperties(PropertyTranslator availableProperties) {
        this.availableProperties.add(availableProperties);
        updateTranslator();
    }
    public void updateTranslator() {    
		//this.availableProperties = availableProperties;
	    propertyTranslator.clear();
	    if (availableProperties ==null) return;
        if (availableProperties.getIdentifiers()!=null) {
            Enumeration e = availableProperties.getIdentifiers().keys();
    	    while (e.hasMoreElements()) {
    	          	Object key = e.nextElement();
    	          	propertyTranslator.put(availableProperties.getIdentifiers().get(key).toString(), key.toString());
            }
        }
        if (availableProperties.getDescriptorProperties()!=null) {
            Enumeration e = availableProperties.getDescriptorProperties().keys();
            while (e.hasMoreElements()) {
                  	Object key = e.nextElement();
                  	propertyTranslator.put(availableProperties.getDescriptorProperties().get(key).toString(), key.toString());
       		}
        }
	}
    
    public int indexOf(Object property,Object value) throws Exception {
        if ((sortedProperties.size()==0) || (!sortedProperties.getPropertyName().equals(property))) {
            sortedProperties.setPropertyName(property);
            
            if (AmbitCONSTANTS.UNIQUE_SMILES.equals(property))  {
                //generate unique smiles
                SmilesGeneratorProcessor g = new SmilesGeneratorProcessor();
                for (int i=0; i < getAtomContainerCount();i++) try {
                    Object c = g.process(getAtomContainer(i));
                    setProperty(i, AmbitCONSTANTS.UNIQUE_SMILES,((IAtomContainer)c).getProperty(AmbitCONSTANTS.UNIQUE_SMILES));
                 } catch (Exception x) {
                     x.printStackTrace();
                 }
                 g = null;
            }
            sortedProperties.addList(this);
        }    
        return sortedProperties.getOriginalIndexOf(value);
    }
    public void sortBy(Object property, boolean ascending) throws Exception {
        if ((sortedProperties.size()==0) || (!sortedProperties.getPropertyName().equals(property))) {
            sortedProperties.clear();
            sortedProperties.setPropertyName(property);
            sortedProperties.addList(this);
        }    
        sortedProperties.sort(ascending);
        //System.out.println(sortedProperties);
    }
    public Collection<Comparable> getValuesPerField(Object field,int limit) {
        ArrayList<Comparable> values = new ArrayList<Comparable>();
        for (int i=0; i < getAtomContainerCount();i++) {
            Object o = getProperty(i, field);
            if (o==null) continue;
            Comparable c=null;
            if (o instanceof Comparable) c = (Comparable)o;
            else c = o.toString();
            if (!values.contains(c)) values.add(c);
            if (values.size()==limit) break;
        }
        return values;
    }
}


