/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

/**
 * 
 */
package chemCast.similarity.data;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.w3c.dom.Element;

import ambit.data.IAmbitEditor;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;

/**
 * @author Nina Jeliazkova
 *
 */
public abstract class Dataset extends ChemCastObject {

    public final static String p_color = "color";
    protected Color color = null;
    protected IAtomContainersList data;
    protected boolean readOnly = true;
	/**
	 * @param element
	 * @throws Exception
	 */
	public Dataset(Element element) throws Exception {
		super(element);
	}

	/**
	 * 
	 */
	public Dataset() {
		super();
	}
	
    public synchronized Color getColor() {
        return color;
    }
    public synchronized void setColor(Color color) {
        firePropertyChange(p_color, this.color, color);
        this.color = color;
    }

	public abstract IAtomContainersList getData() throws Exception;

	public abstract void setData(IAtomContainersList data) throws Exception;
    
    @Override
    public void clear() {
        if (data != null) data.close();
        try {
            setData(null);
        } catch (Exception x) {
            x.printStackTrace();
        }
        super.clear();
    }

	public boolean isReadOnly() {
		return readOnly;
	}

	public void setReadOnly(boolean readOnly) {
		this.readOnly = readOnly;
	}
    public static String Color2Hex(Color c) {
        return
            Integer.toHexString(0x100 | c.getRed()).substring(1)+
            Integer.toHexString(0x100 | c.getGreen()).substring(1)+
            Integer.toHexString(0x100 | c.getBlue()).substring(1);
    }
    public static Color Hex2Color(String clr) throws Exception {
        return  new Color(
                Integer.valueOf(clr.substring(0,2), 16),
                Integer.valueOf(clr.substring(2,4), 16),
                Integer.valueOf(clr.substring(4,6), 16));
    }
    @Override
    public String toString() {
        try {
        if (getData() != null) return getData().toString();
        else return "Dataset";
        } catch (Exception x) {
            return "Dataset";
        }
    }
    @Override
    public IAmbitEditor getEditor() {
        return new DatasetEditor("Dataset",this);
    }
    
}


class DatasetEditor extends JPanel implements IAmbitEditor , ActionListener, FocusListener{
    /**
     * 
     */
    private static final long serialVersionUID = -4881599583581574237L;
    protected Dataset object;
    protected JFormattedTextField nameField;
    protected JFormattedTextField color;
    protected String title;
    public DatasetEditor(String title, Dataset object) {
        super();
        this.title = title;
        this.object = object;
        setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
        JLabel l = new JLabel("Name");
        l.setAlignmentX(JLabel.LEFT_ALIGNMENT);
        //l.setHorizontalTextPosition(JLabel.LEFT);	
        add(l);
        nameField = new JFormattedTextField(object.toString());
        nameField.setToolTipText("Dataset name = "+ object.toString());
        nameField.addActionListener(this);
        nameField.addFocusListener(this);
        nameField.setActionCommand("name");
        l.setLabelFor(nameField);
        add(nameField);
        color = new JFormattedTextField("");
        color.setText(Dataset.Color2Hex(object.getColor()));
        color.setEditable(false);
        color.setOpaque(true);
        color.setBackground(object.getColor());
        color.addMouseListener(new MouseAdapter() {
        	@Override
        	public void mouseClicked(MouseEvent e) {
        		editColor();
        	}
        });
        
        color.setPreferredSize(new Dimension(200,24));
        color.setMinimumSize(new Dimension(200,24));
        color.setToolTipText("Color");
        color.addActionListener(this);
        color.setActionCommand("color");
        add(new JLabel("Color"));
        add(color);
        setBorder(BorderFactory.createTitledBorder(title));

    }
    protected void editColor() {

    	JColorChooser cc = new JColorChooser(object.getColor());
    	if (JOptionPane.showConfirmDialog(this, cc, "", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null) == JOptionPane.OK_OPTION) {
    		object.setColor(cc.getColor());
    		color.setBackground(object.getColor());
    		color.setText(Dataset.Color2Hex(object.getColor()));
    	}
    }
    public void setEditable(boolean editable) {
        nameField.setEditable(editable);
        
    }
    public JComponent getJComponent() {
        return this;
    }
    public boolean view(Component parent, boolean editable, String title) throws AmbitException {
        setEditable(editable);
        return JOptionPane.showConfirmDialog(parent, getJComponent(),title,JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE) == JOptionPane.OK_OPTION;
    }
    public void actionPerformed(ActionEvent e) {
    	if (e.getActionCommand().equals("color")) {
    		object.setColor(JColorChooser.showDialog(this, "Select color", object.getColor()));
    	}
    	
    }
    public void focusLost(FocusEvent e) {
    	
    }
    public void focusGained(FocusEvent e) {
    	// TODO Auto-generated method stub
    	
    }
    public boolean isEditable() {
    	return nameField.isEditable();
    }
}
