/* SubsetsXYPlot.java
 * Author: Nina Jeliazkova
 * Date: Nov 12, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SymbolAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeriesCollection;
import org.openscience.cdk.tools.SaturationChecker;

import chemCast.similarity.data.Subsets;

public class SubsetsXYPlot extends XYPlot implements PropertyChangeListener {
    protected LogarithmicAxis logRangeAxis=null;
    protected NumberAxis rangeAxis=null;
    protected SymbolAxis symbolAxis=null;
    protected boolean logAxis = false;
    protected boolean categoryAxis = false;
    /**
     * 
     */
    private static final long serialVersionUID = -4537114390714206112L;


    public SubsetsXYPlot(MoleculesXYZDataset[] dataset, XYItemRenderer[] renderer) {
        super(new XYSeriesCollection(), 
                new NumberAxis(dataset[0].getPropertyX().toString()),
                new NumberAxis(dataset[0].getPropertyY().toString()),
                new XYLineAndShapeRenderer());
        for (int i=0; i < dataset.length;i++) {
            setDataset(i+1, dataset[i]);
            mapDatasetToDomainAxis(i+1, 0);
            mapDatasetToRangeAxis(i+1, 0);
            setRenderer(i+1, renderer[i]);       
        }    
        rangeAxis = (NumberAxis)getRangeAxis();
        //setDataset(0, null);
        //setRenderer(0, null);

    }
    public void propertyChange(PropertyChangeEvent evt) {
        
        if (evt.getPropertyName().equals(Subsets.p_groups))
        for (int i=1; i < getDatasetCount(); i++) {
            DatasetChangeEvent event = new DatasetChangeEvent(this, getDataset(i));
            datasetChanged(event);
        }
        
    }
    public Object getProperty(int index) {
    	for (int i=0; i < getDatasetCount();i++) {
    		XYDataset d = getDataset(i);
    		try {
    		if (d instanceof MoleculesXYZDataset) 
		        switch (index) {
		        case 0: { return ((MoleculesXYZDataset)d).getPropertyX();}
		        case 1: { return ((MoleculesXYZDataset)d).getPropertyY();}
		        }
    		} catch (Exception x) {
    			return null;
    		}
        }
    	return null;
    	
    }
    public synchronized void setProperty(int index, Object property) {
    	for (int i=0; i < getDatasetCount();i++) {
    		XYDataset d = getDataset(i);
    		try {
    		if (d instanceof MoleculesXYZDataset) 
		        switch (index) {
		        case 0: { ((MoleculesXYZDataset)d).setPropertyX(property); break;}
		        case 1: { ((MoleculesXYZDataset)d).setPropertyY(property); break;}
//		        case 2: { ((MoleculesXYZDataset)d).setPropertyZ(property); break;}
                case 2: {
                    ((MoleculesXYZRenderer)getRenderer(i)).setColorProperty(property);
                    ((MoleculesXYZDataset)d).setColorProperty(property);
                    break;
                    }
                case 3: {
                    
                    ((MoleculesXYZDataset)d).setLabelProperty(property);
                    break;
                    }
                    
		        }
    		} catch (Exception x) {
    			x.printStackTrace();
    		}
        }
    }
	public boolean isLogAxis() {
		return logAxis;
	}
	public void setLogAxis(boolean logAxis) {
		this.logAxis = logAxis;
		if (logAxis) {
			if (logRangeAxis==null) {
				logRangeAxis = new LogarithmicAxis(rangeAxis.getLabel());
				logRangeAxis.setAutoRange(true);
				//logRangeAxis.setLog10TickLabelsFlag(true);
				logRangeAxis.setExpTickLabelsFlag(true);
			}
			logRangeAxis.setAllowNegativesFlag(true);
			logRangeAxis.setLabel(rangeAxis.getLabel());
			setRangeAxis(logRangeAxis);
		} else {
			if (logRangeAxis != null) {
				rangeAxis.setLabel(logRangeAxis.getLabel());
			}
			setRangeAxis(rangeAxis);
		}
	}
	public boolean isCategoryAxis() {
		return categoryAxis;
	}
	public void setCategoryAxis(boolean categoryAxis) {
		this.categoryAxis = categoryAxis;
		if (categoryAxis) {
			if (symbolAxis==null) {
				symbolAxis = new SymbolAxis(rangeAxis.getLabel(),new String[] {});
				symbolAxis.setAutoRange(true);
			}
			symbolAxis.setLabel(rangeAxis.getLabel());
			setRangeAxis(symbolAxis);
		} else {
			if (symbolAxis != null) {
				rangeAxis.setLabel(symbolAxis.getLabel());
				logRangeAxis.setLabel(symbolAxis.getLabel());
			}
			setRangeAxis(rangeAxis);
		}		
	}
	

    

}
