/* FileBatchProcessing.java
 * Author: Nina Jeliazkova
 * Date: Oct 17, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui.actions;

import javax.swing.Icon;
import javax.swing.JFrame;

import org.openscience.cdk.qsar.IMolecularDescriptor;

import ambit.data.descriptors.DescriptorsHashtable;
import ambit.io.FastTemplateHandler3D;
import ambit.io.batch.IBatchStatistics;
import ambit.processors.Builder3DProcessor;
import ambit.processors.IAmbitProcessor;
import ambit.processors.IdentifiersProcessor;
import ambit.processors.ProcessorsChain;
import ambit.processors.descriptors.CalculateDescriptors;
import ambit.processors.descriptors.MopacProcessor;
import ambit.processors.descriptors.MopacShell;
import ambit.ui.UITools;
import ambit.ui.actions.file.FileBatchProcessingAction;
import chemCast.application.ChemCASTModule;
import chemCast.similarity.data.ChemCastDescriptorsList;

/**
 * TODO add description
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Oct 17, 2006
 */
public class FileBatchProcessing extends FileBatchProcessingAction {

    /**
     * 
     */
    private static final long serialVersionUID = -6940376464843936800L;
    /**
     * @param userData
     * @param mainFrame
     */
    public FileBatchProcessing(ChemCASTModule userData, JFrame mainFrame) {
        this(userData, mainFrame,"Batch");
    }

    /**
     * @param userData
     * @param mainFrame
     * @param name
     */
    public FileBatchProcessing(ChemCASTModule userData, JFrame mainFrame, String name) {
        this(userData, mainFrame, name,UITools.createImageIcon("ambit/ui/images/batch.png"));
    }

    /**
     * @param userData
     * @param mainFrame
     * @param name
     * @param icon
     */
    public FileBatchProcessing(ChemCASTModule userData, JFrame mainFrame, String name,
            Icon icon) {
        super(userData, mainFrame, name, icon);
        // TODO Auto-generated constructor stub
    }
    ChemCASTModule getChemCastData() {
        return (ChemCASTModule) userData;
    }
    public IAmbitProcessor getProcessor() {
        ProcessorsChain processors = new ProcessorsChain();
        identifiersProcessor = new IdentifiersProcessor();
        //processors.addProcessor(identifiersProcessor);
        ChemCastDescriptorsList d = ChemCASTModule.readDescriptors();
        boolean runMopac = false;
        boolean runBuilder = false;
        DescriptorsHashtable descriptors = new DescriptorsHashtable();
        for (int i=d.getList().size()-1;i>=0;i--) {
            if (d.get(i).isRequired3D()) runBuilder = true;
            if (d.get(i).getCalculator() instanceof MopacShell) {
                runMopac = true;
                d.getList().remove(i);
            }
            if (d.get(i).isEnabled()) {
                IMolecularDescriptor c = d.get(i).getCalculator();
                if (c!=null)
                    descriptors.addDescriptorPair(c, d.get(i));
            }    
        }
        d.clear();
        d = null;
        
        FastTemplateHandler3D templateHandler = FastTemplateHandler3D.getInstance();
        if (runBuilder) processors.addProcessor(new Builder3DProcessor(templateHandler));
        MopacProcessor mp = new MopacProcessor(new MopacShell());
        mp.setInteractive(false);
        if (runMopac) processors.addProcessor(mp);
        
        if (descriptors.size()>0) {
            CalculateDescriptors calculator = new CalculateDescriptors(descriptors,templateHandler) {
            	public void setProperty(org.openscience.cdk.interfaces.IAtomContainer ac, Object key, Object value) {
            		
            		ac.setProperty(key.getClass().getName(), value);            		
            	};
            };
            processors.addProcessor(calculator);
        }
        return processors;
    }
    public IBatchStatistics getBatshStatistics() {
        IBatchStatistics bs = getChemCastData().getBatchStatistics();
        
        bs.setResultCaption("Processed ");
        return bs;
    }

}
