/*
Copyright Ideaconsult Ltd. (C) 2005-2007  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package chemCast.ui.actions;

import java.io.File;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.JFrame;

import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import ambit.io.FileInputState;
import ambit.io.MolPropertiesIOSetting;
import ambit.io.MyIOUtilities;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Endpoint;
import chemCast.similarity.data.Endpoints;
import chemCast.similarity.data.SimilarityDef;
import chemCast.similarity.data.SimilarityDefs;

public class TrainingSetOpenAction extends FileOpenAction {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1010489090812712571L;
	
	public TrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            int dataset) {
        this(userData, mainFrame, "Open", dataset);
    }

    public TrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, int dataset) {
        this(userData, mainFrame, name, icon, dataset);
    }

    public TrainingSetOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, Icon icon, int dataset) {
        super(userData, mainFrame, name, icon, dataset);
        putValue(SHORT_DESCRIPTION,"Open training data sets");
    }
    public DatasetDef getDataset() throws SubsetException {
    	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading endpoints");
        Endpoints endpoints = ChemCASTModule.readEndpoints();
        int index = endpoints.indexOf(((ChemCASTModule)userData).getEndpoint());
        subsets.clear();
        Endpoint endpoint = endpoints.findByName("New");
        if (endpoint != null) {
            ((ChemCASTModule) userData).setEndpoint(endpoint);
            //File file = new File(wizard.getEndpoint().getSource().getDatasetValue());
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,endpoint.getTrainingData()+" selected");
            //wizard.getEndpoint().getTrainingData().setColor(Color.blue);
            DatasetDef dataset = endpoint.getTrainingData();
            if (dataset.getDatasetSource().equals(DatasetDef.p_filesource)) {
                File file = new File(dataset.getDatasetValue());
                if (!file.exists()) {
                    file = MyIOUtilities.selectFile(mainFrame,null,
                            getChemCastData().getDefaultDir(),
                            FileInputState.extensions,FileInputState.extensionDescription,true);
                    if (file != null) 
                        dataset.setDatasetValue(file.getAbsolutePath());
                    else return null;
                }    
            }
            return endpoint.getTrainingData();
            
        } else {
        	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null," ");
        	return null;
        }

    }    
    protected void updateEndpoint(Endpoint endpoint, DatasetDef dataset) {
    	SimilarityDefs similarityMethods = endpoint.getSimilarityMethods();
    	ChemCastDescriptorsList d = dataset.getDescriptorsList();
    	
    	for (int i=0; i < similarityMethods.getList().size(); i++) {
    		SimilarityDef method = similarityMethods.get(i);
    		
    		//assign descriptor names
    		if ("chemCast.weka.WekaSimilarityModel".equals(method.getClassName()) ||
    				"chemCast.weka.WekaClustering".equals(method.getClassName())) {
		    		method.getDescriptors().clear();
		    		
		    		System.out.println(method.getClassName());
		    		for (int j=0; j < d.getList().size();j++)
		    			method.getDescriptors().add(d.get(j));
		    		   		
    		}
    	}
    }
    protected IReaderListener getReaderListener() {

        return new IReaderListener() {
          public void frameRead(ReaderEvent e) {
              getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
              //getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading record");
          }
          public void processIOSettingQuestion(IOSetting setting) {
              if (setting instanceof MolPropertiesIOSetting) {
                  dataset.setProperties(((MolPropertiesIOSetting)setting).getProperties());
                  //System.out.println(dataset.getProperties());
                  
              }  
          }
        };
    }    
    
}


