/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.tree.cramer;

import junit.framework.TestCase;

import org.openscience.cdk.Molecule;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.isomorphism.UniversalIsomorphismTester;

import toxTree.core.IDecisionRule;
import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.MolAnalyseException;
import toxTree.logging.TTLogger;
import toxTree.query.FunctionalGroups;
import toxTree.query.MolAnalyser;
import toxTree.tree.cramer.RuleAliphaticWithSomeFuncGroups;

/**
 * TODO add description
 * @author Vedina
 * <b>Modified</b> 2005-8-19
 */
public class RuleAliphaticWithSomeFuncGroupsTest extends TestCase {

	public static void main(String[] args) {
		junit.textui.TestRunner.run(RuleAliphaticWithSomeFuncGroupsTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for RuleAliphaticWithSomeFuncGroupsTest.
	 * @param arg0
	 */
	public RuleAliphaticWithSomeFuncGroupsTest(String arg0) {
		super(arg0);
		TTLogger.configureLog4j(true);
	}
	public void testRule20() {
	    IDecisionRule rule20 = new RuleAliphaticWithSomeFuncGroups();
	    //
	    Object[][] answer = {
	            {"CC(=O)CCC(C)=O",new Boolean(false)}, //20N
	            {"CSCCCNCS",new Boolean(false)}, //20N
	            {"O1C(C)=CC(=O)C(C(=O)C)C(=O)1",new Boolean(false)}, //20N
	            {"CC(=O)C(O)=O",new Boolean(true)},  //20Y
	            {"C=C(C(=O)O)C(=O)O",new Boolean(true)},  //20Y
	            {"CCCC(=O)SC",new Boolean(true)},  //20Y
	            {"CCOCCOCCOCCCCCCCCCCCC",new Boolean(true)},  //SLTS unsulphated"
				{"CN(C)C(=S)SCSC(=S)N(C)C",new Boolean(true)}  //fails (Munro ) 
	            };
        
	    for (int i = 7; i < answer.length; i++) {
	        Molecule mol = (Molecule) FunctionalGroups.createAtomContainer((String)answer[i][0]);
	        if (mol != null)
	            try {
	            	System.out.println(answer[i][0]);
	            	MolAnalyser.analyse(mol);
	                Boolean b = (Boolean) answer[i][1];
	                assertEquals(b.booleanValue(),rule20.verifyRule(mol));
	            } catch (DecisionMethodException x) {
	                x.printStackTrace();
	            } catch (MolAnalyseException x) {
	                x.printStackTrace();
	            }
	    }
	    
	}
	
	public void testBugOverlapingGroups() {
	    IDecisionRule rule20 = new RuleAliphaticWithSomeFuncGroups();
	    //two smiles of the same compound
	    Object[][] answer = {
	            {"COCCOC(=O)C=C",new Boolean(true)}, 
	            {"C=CC(=O)OCCOC",new Boolean(true)} 
	            
	            };
	    //1N,2N,3N,5N,6N,7N,16N,17N,19Y,20Y,21N,18N        
	    Molecule mol1 = (Molecule) FunctionalGroups.createAtomContainer((String)answer[0][0]);
	    Molecule mol2 = (Molecule) FunctionalGroups.createAtomContainer((String)answer[1][0]);
	            try {
	        	    assertTrue(UniversalIsomorphismTester.isIsomorph(mol1,mol2));
	            	
	            	MolAnalyser.analyse(mol1);
	            	MolAnalyser.analyse(mol2);
	                assertTrue(rule20.verifyRule(mol1));
	            	assertTrue(rule20.verifyRule(mol2));
	            	
	            } catch (DecisionMethodException x) {
	                x.printStackTrace();
	            } catch (MolAnalyseException x) {
	                x.printStackTrace();
	            } catch (CDKException x) {
	            	x.printStackTrace();
	            }
	    
	}
	
		
	

}
