/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.tree.rules.smarts;


import java.util.Enumeration;
import java.util.Hashtable;

import joelib.molecule.JOEMol;
import joelib.smarts.JOESmartsPattern;

import org.openscience.cdk.Molecule;

import toxTree.core.IDecisionRuleEditor;
import toxTree.exceptions.DecisionMethodException;
import toxTree.tree.AbstractRule;
import toxTree.ui.tree.rules.SMARTSRuleEditor;

public class RuleSMARTSubstructure  extends AbstractRule implements IRuleSMARTSubstructures    {
	private static final long serialVersionUID = 0;
	protected  Hashtable<String,SmartsPattern> smartsPatterns;
	protected  boolean containsAllSubstructures = false;
	public RuleSMARTSubstructure() {
		super();	
        smartsPatterns = new Hashtable<String,SmartsPattern>();
		explanation = new StringBuffer();
		explanation.append("Returns true if the query contains substructures specified by SMARTS patterns.");
		setTitle("Substructure by SMARTS patterns.");
		setID("SMARTS");

	}
	public void addSubstructure(String title, String smarts) throws SMARTSException {
		if ("".equals(title)) addSubstructure(smarts);
		else 
			smartsPatterns.put(title, new SmartsPattern(smarts));
		
	}
	public void addSubstructure(String smarts) throws SMARTSException{	
		addSubstructure(Integer.toString(smartsPatterns.size()+1),smarts);
	}
	public void setSubstructure(String title,String smarts) throws SMARTSException{
		smartsPatterns.put(title, new SmartsPattern(smarts));
	}
    public void clearSubstructures() {
    	smartsPatterns.clear();
    }

    public void deleteSubstructure(String title) {
    	smartsPatterns.remove(title);
    }	
	public boolean  verifyRule(org.openscience.cdk.interfaces.IAtomContainer mol) throws DecisionMethodException {
				
		JOEMol joemol = new JOEMol();
		joemol = Convertor.convert((Molecule)mol);
		Enumeration e  = smartsPatterns.keys();
		boolean is_true = false;
		String temp_id = "";
    	while(e.hasMoreElements()){
    		temp_id = e.nextElement().toString();
    		
    		SmartsPattern pattern = smartsPatterns.get(temp_id);
            if (pattern == null)
            {
            	throw new DecisionMethodException("ID '" + id + "' is missing in " +
            			getClass().getName());
            }
            
    		is_true = pattern.hasSMARTSPattern(joemol)>0;
    		
    		if(containsAllSubstructures && !is_true){
    			
    			return false;
    		}
    		else if(!containsAllSubstructures && is_true){    			
    			return true;
    		}
    		
    		
    		
    	}
    	return is_true;
	}
	public void removeSingleSMARTS(Hashtable table, String id) throws SMARTSException
        {
            
            if (!table.containsKey(id))
            {
            	throw new SMARTSException("Invalid(id:" + id + ") '" +
                      "' defined in " + getClass().getName());
            }

            table.remove(id);
        }

	public void initSingleSMARTS(Hashtable<String,SmartsPattern> table, String id,   String smartPattern) throws SMARTSException 
    {
		SmartsPattern smarts = new SmartsPattern(smartPattern);
        table.put(id, smarts);
    }
    @Override
    public boolean isImplemented() {
    	return (smartsPatterns!=null) && (smartsPatterns.size()>0);
    }
	
	@Override
	public IDecisionRuleEditor getEditor() {
		return new SMARTSRuleEditor(this);
	}
	public boolean containsAllSubstructures() {
		return containsAllSubstructures;
	}
	public void setContainsAllSubstructures(boolean allSmarts) {
		this.containsAllSubstructures = allSmarts;
	}
	public Hashtable getSmartsPatterns() {
		return smartsPatterns;
	}

	public void setSmartsPatterns(Hashtable<String, SmartsPattern> smartsPatterns) {
		this.smartsPatterns = smartsPatterns;
	}	

}

