/*
Copyright Ideaconsult Ltd. (C) 2005-2007  
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/


package toxTree.tree.rules.smarts;

import java.io.Serializable;

import joelib.molecule.JOEMol;
import joelib.smarts.JOESmartsPattern;

/**
 * JoeSmartsPattern is hidden, mainly to avoid some problems in XML serialization.
 * @author Nina Jeliazkova
 *
 */
public class SmartsPattern implements Serializable{
	/**
     * 
     */
    private static final long serialVersionUID = 1149358670252272266L;
    protected String smarts = "";
	protected transient JOESmartsPattern joeSmartsPatern = null;
	public SmartsPattern() {
		smarts = "";
		joeSmartsPatern = null;
	}
	public SmartsPattern(String smarts) throws SMARTSException {
		setSmarts(smarts);
	}
	public String getSmarts() {
		return smarts;
	}
	public void setSmarts(String smarts) throws SMARTSException {
		joeSmartsPatern = null;
		joeSmartsPatern = new JOESmartsPattern();
        if (!joeSmartsPatern.init(smarts))
        	throw new SMARTSException("Invalid SMARTS pattern " +  smarts + ")");
		this.smarts = smarts;
	}
    public int hasSMARTSPattern(JOEMol mol) throws SMARTSException
    {	
    	if (joeSmartsPatern == null) throw new SMARTSException("Undefined SMARTS pattern");
        joeSmartsPatern.match(mol);
        return joeSmartsPatern.numMatches();
    }
    @Override
    public String toString() {
    	return smarts;
    }
}
